﻿using System;
using System.IO;
using System.Linq;

using NAudio.Wave;

namespace ENCODER;

/// <summary>
/// 인코더
/// </summary>
class Encoder
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Static
    //////////////////////////////////////////////////////////////////////////////// Private

    #region 프로그램 시작하기 - Main(argumentArray)

    /// <summary>
    /// 프로그램 시작하기
    /// </summary>
    /// <param name="argumentArray">인자 배열</param>
    private static void Main(string[] argumentArray)
    {
        if(argumentArray.Length != 2)
        {
            Console.WriteLine("사용법 : encoder.exe source.zip target.wav");

            return;
        }

        string sourceFilePath = argumentArray[0];
        string targetFilePath = argumentArray[1];

        // 소스 파일을 읽는다.
        byte[] sourceFileByteArray = File.ReadAllBytes(sourceFilePath);

        int sourceFileSize = sourceFileByteArray.Length;

        // 바이트를 비트로 변환한다.
        string dataBitString = string.Join(string.Empty, sourceFileByteArray.Select(sourceByte => Convert.ToString(sourceByte, 2).PadLeft(8, '0')));

        // 파일 크기를 헤더에 추가한다. (32비트로 표현)
        string fileSizeBitString = Convert.ToString(sourceFileSize, 2).PadLeft(32, '0');

        // 시작 및 종료 마커 문자열을 설정한다.
        string startMarkerString = "1111000011110000";
        string endMarkerString   = "0000111100001111";

        // 전체 비트 스트림을 생성한다.
        string bitStreamString = startMarkerString + fileSizeBitString + dataBitString + endMarkerString;

        // 파라미터를 설정한다.
        int    samplingFrequency = 44100; // 샘플링 주파수
        double durationPerBit    = 0.01;  // 각 비트의 지속 시간 (초)
        int    sampleCountPerBit = (int)(samplingFrequency * durationPerBit);

        int frequency0 = 1000; // '0'에 대한 주파수
        int frequency1 = 2000; // '1'에 대한 주파수

        // 신호를 생성한다.
        float[] signalArray = new float[bitStreamString.Length * sampleCountPerBit];

        int bitStreamStringLength = bitStreamString.Length;

        for(int i = 0; i < bitStreamStringLength; i++)
        {
            char bitCharacter = bitStreamString[i];

            int frequency = bitCharacter == '1' ? frequency1 : frequency0;

            for(int j = 0; j < sampleCountPerBit; j++)
            {
                double t = (double)j / samplingFrequency;

                signalArray[i * sampleCountPerBit + j] = (float)Math.Sin(2 * Math.PI * frequency * t);
            }
        }

        // 신호를 정규화한다.
        float maximumAmplitude = signalArray.Max(x => Math.Abs(x));

        for(int i = 0; i < signalArray.Length; i++)
        {
            signalArray[i] = signalArray[i] / maximumAmplitude;
        }

        // WAV 파일로 저장한다.
        using(WaveFileWriter waveFileWriter = new WaveFileWriter(targetFilePath, new WaveFormat(samplingFrequency, 16, 1)))
        {
            waveFileWriter.WriteSamples(signalArray, 0, signalArray.Length);
        }

        Console.WriteLine("인코딩 완료 : " + targetFilePath);
    }

    #endregion
}